import opensimplex
import pygame
import sys
import time

WHITE = (255,255,255)
BLACK = (0,0,0)
DARK_BROWN = (75, 44, 17)
GREEN = (38, 119, 9)
BROWN = (161, 74, 12)
BLUE = (59, 224, 227)

block_width = 16
block_height = 16
block_depth = block_height/2

generator = opensimplex.OpenSimplex()

pygame.init()
screen = pygame.display.set_mode((700, 300))

def pygame_drawing(func):
    def func_wrapper(*args, **kwargs):
        screen.fill(BLACK)
        func(*args, **kwargs)
        pygame.display.update()
        while True:
            for event in pygame.event.get():
                if event.type==pygame.QUIT:
                    pygame.quit()
                    sys.exit()
        
        return func
    return func_wrapper

@pygame_drawing
def render_heightmap(height_map):
    height = len(height_map)
    width = len(height_map[0])

    delta = max(map(max, height_map))

    for y in range(height):
        for x in range(width):
            rgb_val = (255/delta)*height_map[y][x]
            color = (rgb_val, rgb_val, rgb_val)
            screen.fill(color, ((x*5, y*5), (5, 5)))

@pygame_drawing
def render_1d_terrain(terrain):
    width = len(terrain)
    delta = max(terrain)

    screen.fill(BLUE)

    for x in range(width):
        for y in range(terrain[x], 60):

            color = BROWN
            if y <= terrain[x]+1:
                color = GREEN

            screen.fill(color, ((x*5, y*5), (5, 5)))

@pygame_drawing
def render_2d_terrain(terrain):
    height = len(terrain)
    width = len(terrain[0])

    delta = max(map(max, terrain))

    for y in range(height):
        for x in range(width):
            value = terrain[y][x]

            if value == 1 and (terrain[y-1][x] == 2):
                color = GREEN
            elif value == 0:
                color = DARK_BROWN
            elif value == 1:
                color = BROWN
            else:
                color = BLUE

            screen.fill(color, ((x*5, y*5), (5, 5)))

@pygame_drawing
def render_3d_terrain(terrain):
    delta = len(terrain)
    height = len(terrain[0])
    width = len(terrain[0][0])

    #camera = Camera(screen, 1, 1, 700, 300, delta, height, width, block_size=16)
    #camera.draw_map(terrain)
    grid_x = 300
    grid_y = 550

    for z in range(delta):
        for x in range(width):
            for y in range(height):
                #print(str(x) + ", " + str(y) + ", " + str(z))

                iso_x = grid_x + ((x-y) * (block_width // 2))
                iso_y = grid_y + ((x+y) * (block_depth // 2)) - (block_depth * (width // 2)) - block_depth*z
                draw(terrain[z][y][x], iso_x, iso_y)



def generate_1d_noise(width, delta, seed):
    height_map = []
    for x in range(width):
        nx = seed + (x/width - 0.5)
        noise = generator.noise2d(nx, 0)/2 + 0.5
        elevation = round(noise*delta)
        height_map.append(elevation)
    return height_map

def draw(tile, x, y):
    if tile == 0: # no tile so don't draw anything
        return
    # Fill tile walls
    pygame.draw.polygon(screen,
                    pygame.Color(150, 75, 0),
                    [(x,y), 
                    (x+block_width/2, y+block_height//4), 
                    (x+block_width/2, y+(block_height//4)+block_height//2), 
                    (x, y+block_height//2)])
    pygame.draw.polygon(screen,
                    pygame.Color(74, 37, 0),
                    [(x+block_width//2, y+block_height//4), 
                    (x+block_width, y), 
                    (x+block_width, y+block_height//2), 
                    (x+block_width//2, y+(block_height//4)+block_height//2)])
    # Fill tile "ceilings"
    pygame.draw.polygon(screen, 
                        pygame.Color("green"),
                        [(x,y), (x+block_width/2, y-block_height//4), (x+block_width, y), (x+block_width//2, y+block_height//4)]
    )


class Camera:

    def __init__(self, screen, foc_x, foc_y, width, height, world_depth, world_height, world_width, block_size=32):
        self.screen = screen
        self.x = foc_x
        self.y = foc_y
        self.width = width
        self.height = height
        self.middle_screen_x = 0# width//2
        self.middle_screen_y = 0#height//2
        self.block_size = block_size
        self.world_depth = world_depth
        self.world_height = world_height
        self.world_width = world_width

    def move(self, delta_x, delta_y):
        if 0 <= self.x+delta_x <= self.world_width:
            self.x += delta_x
        if 0 <= self.y+delta_y <= self.world_height:
            self.y += delta_y

    def get_screen_coordinates(self, x, y, z):

        spx = (x - (self.x - self.middle_screen_x))
        spy = (y - (self.y - self.middle_screen_y))
        
        spx = min(x, spx)
        if self.x + (self.block_size//2) > self.world_width:
            spx += (self.block_size//2)+(self.x-self.world_width)

        spy = min(y, spy)
        if self.y + (self.block_size//2) > self.world_height:
            spy += (self.block_size//2)+(self.y-self.world_height)

        return spx*self.block_size, spy*self.block_size
    
    def draw(self, tile, x, y):
        if tile == 0: # no tile so don't draw anything
            return
        # Fill tile walls
        pygame.draw.polygon(self.screen,
                        pygame.Color(150, 75, 0),
                        [(x,y), 
                        (x+self.block_size//2, y+self.block_size//4), 
                        (x+self.block_size//2, y+(self.block_size//4)+self.block_size//2), 
                        (x, y+self.block_size//2)])
        pygame.draw.polygon(self.screen,
                        pygame.Color(74, 37, 0),
                        [(x+self.block_size//2, y+self.block_size//4), 
                        (x+self.block_size, y), 
                        (x+self.block_size, y+self.block_size//2), 
                        (x+self.block_size//2, y+(self.block_size//4)+self.block_size//2)])
        # Fill tile "floors"
        pygame.draw.polygon(self.screen, 
                            pygame.Color("green"),
                            [(x,y), (x+self.block_size//2, y-self.block_size//4), (x+self.block_size, y), (x+self.block_size//2, y+self.block_size//4)])

    def draw_map(self, tiles):
        for z in range(self.world_depth):
            for x in range(max(0, self.x), min(self.x+32, self.world_width)):
                for y in range(max(0, self.y), min(self.y+32, self.world_height)):
                    spx, spy = self.get_screen_coordinates(x, y, z)

                    iso_x = (self.x*self.block_size) + self.middle_screen_x + ((x-y) * (self.block_size // 2))
                    iso_y = (self.y*self.block_size) + self.middle_screen_y +((x+y) * (self.block_size // 4)) - ((self.block_size//2) * (self.world_width // 2)) - (self.block_size//2)*z

                    self.draw(tiles[z][y][x], iso_x, iso_y)